use super::Symbol;
use crate::span::Span;

/// The type of a [Token].
#[derive(Debug, Clone, PartialEq)]
pub enum TokenKind {
    /// Block comment, e.g. '#| ... |#'
    BlockComment(String),
    /// Line comment, e.g. '; ...'
    LineComment(String),

    /// Opening parenthesis, e.g. '('
    OpenParen,
    /// Closing parenthesis, e.g. ')'
    CloseParen,
    /// Opening brace, e.g. '{'
    OpenBrace,
    /// Closing brace, e.g. '}'
    CloseBrace,
    /// Opening bracket, e.g. '['
    OpenBracket,
    /// Closing bracket, e.g. ']'
    CloseBracket,
    /// Opening hash-brace, e.g. '#{'
    OpenHashBrace,

    /// Boolean, e.g. 'true', 'false'
    Bool(bool),
    /// Character, e.g. 'c', '\n'
    Char(char),
    /// Number, e.g. '1', '2.0', '0.003'
    Number(f64),
    /// String, e.g. '"foo bar"'
    String(String),
    /// Keyword, e.g. ':baz'
    Keyword(Symbol),
    /// Symbol, e.g. 'qux', '+'
    Symbol(Symbol),
    /// Nil, e.g. 'nil'
    Nil,
}

/// A token with a start and end location.
#[derive(Debug, Clone, PartialEq)]
pub struct Token {
    /// The type of token.
    pub kind: TokenKind,
    /// The span in which the token occurs.
    pub span: Span,
}

impl Token {
    /// Construct a new instance of `Token`.
    #[must_use]
    pub const fn new(kind: TokenKind, span: Span) -> Self {
        Self { kind, span }
    }
}
