use crate::{lexer::LexerError, span::Span};

/// Errors that can occur during parsing.
#[derive(Debug, Clone, PartialEq)]
pub enum ParserErrorKind {
    /// An error which ocurred during lexical analysis.
    Lexer(LexerError),
    /// An unexpecting closing parenthesis/bracket was encountered.
    UnexpectedCloseBracket,
    /// Opening parenthesis/bracket does not have a matching closing
    /// parenthesis/bracket.
    UnclosedBracket,
    /// An unmatched parenthesis/bracket was encountered.
    UnmatchedBracket,
    /// Unexpected parser state reached.
    Unreachable,
}

/// Parser error, with a start and end location.
#[derive(Debug, Clone, PartialEq)]
pub struct ParserError {
    /// The type of error encountered.
    pub kind: ParserErrorKind,
    /// The span in which the error occurred.
    pub span: Span,
}

impl ParserError {
    /// Construct a new instance of `ParserErorr`.
    #[must_use]
    pub const fn new(kind: ParserErrorKind, span: Span) -> Self {
        Self { kind, span }
    }
}

impl std::error::Error for ParserError {}

impl std::fmt::Display for ParserError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        use ParserErrorKind::*;

        match &self.kind {
            Lexer(err) => write!(f, "{err}"),
            UnexpectedCloseBracket => write!(f, "Unexpected closing bracket"),
            UnclosedBracket => write!(f, "Unclosed parenthesis"),
            UnmatchedBracket => write!(f, "Unmatched bracket"),
            Unreachable => write!(f, "Unexpected parsing state reached"),
        }
    }
}

impl From<LexerError> for ParserError {
    fn from(err: LexerError) -> Self {
        let span = err.span.clone();
        Self::new(ParserErrorKind::Lexer(err), span)
    }
}
